﻿/**
 * version: 1.3
 * date: 05/23/2010
 * Actionscript 2.0
 * Flash Player 8
 * @author Ahmed Abbas, ahmed.abbas@ixd-consultant.com
 * @contributors
		Jan Jonas
			Leiter Softwareentwicklung
			learnbit Internet und Multimedia GmbH
			Buchwiese, Germany
		Yasmin
			aka geekgirl
 *
 * requirements:
 
 	1. dynamic TextField.
	2. HTML enabled TextField.
	3. pre-assigned TextFormat.
	4. Arabic fonts.
 
 *
 * notes:
 
 	1. for embedding fonts, when using StyleSheet it must be assigned after calling this method and after setting TextFormat for proper fornt detection, for example:
	
		import com.xvisage.utils.StringUtils;
		var utils:StringUtils = new StringUtils();
		// "output" is a TextField on-stage
		output.htmlText = utils.parseArabic("ARABIC TEXT", output);
		var format:TextFormat = new TextFormat();
		format.color = 0x0066FF;
		output.setTextFormat(format);
		var styles:StyleSheet = new StyleSheet();
		styles.setStyle("a:hover", {color:"#000099"});
		output.stylesheet = styles;
	2. for matching/comparing input string, when referencing original input string use the data property (utils.data) instead of (TextField.text) where "utils" is an instance of StringUtils class.
 
 *
 * features supported:
 
 	1. embedding fonts (just put a dynamic textfield on-stage and select at least Basic Latin (95 glyphs) and Arabic (1088 glyphs) from the Character Embedding menu).
	2. Arabic ligatures.
	3. word wrapping.
	4. bi-directional text.
	5. HTML Text.
	6. loading external text on run-time.
	7. Windows/Mac/Linux support.
	8. Arabic enabled input fields using actionscript method (createArabicInput) with the help of javascript (arabicinput.js).
 
 *
 * features not supported:
 
 	1. Arabic diacritics (not necessarily for reading Arabic language in general, arabs are using diacritics mostly for writing quran, and in that case diacritics are not the only missing feature, there're many other ligatures cases in Arabic used for quran writing).
 
 *
 * change list:
 
 	1. new approach for wrapping text properly involving HTML tags, brackets, punctuation marks and bi-directional cases.
	2. looping through input string thoroughly only once for better performance.
	3. object class model structure with public properties and methods instead of static parser.
 
 *
 * fixed bugs:
 
 	1. Arabic character condition for letter (ة).
	2. beginning and ending text lines with non-Arabic characters (Embedding fonts required for Mac).
	3. Inline HTML tags (Embedding fonts required for Mac).
	4. Symmetrical swapping for bi-directional brackets.
 
 * known bugs:
 
 	1. improper wrapping for non-embedded fonts only on Mac when begin text line with non-Arabic characters or when using custom format HTML tags.
 
 *
 * example:
 	
	import com.xvisage.utils.StringUtils;
	var utils:StringUtils = new StringUtils();
	
	var format:TextFormat = new TextFormat();
	format.font = "Arial";
	format.size = 14;
	format.align = "right";
	
	var output:TextField = this.createTextField("output", 1, 10, 10, Stage.width-20, format.size);
	output.autoSize = true;
	output.embedFonts = true;
	output.wordWrap = true;
	output.multiline = true;
	output.html = true;
	output.text = " "; // initial charachter to be able to apply TextFormat (not required if TextField instance already on-stage)
	output.setTextFormat(format); // required in order to detect proper format
	
	var xml:XML = new XML();
	xml.ignoreWhite = true;
	xml.onLoad = function(done:Boolean) {
		if (done) {
			output.htmlText = utils.parseArabic(this.firstChild.firstChild.nodeValue, output);
			output.setTextFormat(format);
		}
	}
	xml.load("arabic.xml");
	
	<?xml version="1.0" encoding="utf-8"?>
	<arabic>
		<![CDATA[الهدف من لعبة الكلمات المتقاطعة هو ملء المربعات البيضاء، وتشكيل الكلمات أو العبارات، عن طريق حل القرائن التي تؤدي إلى إجابات. والمربعات السوداء تستخدم لفصل الكلمات أو العبارات. ويوضع لكل صف أو عمود رقم، ثم يكتب أمام الرقم ما يشابه الكلمة المطلوب كتابتها في المربعات، وتحوي المجلات أنواعا من هذه اللعبة؛ فمنها سهل وآخر صعب، والكلمات المتقاطعة في مجلة معينة تتبع أسلوب يختلف عن الأخرى، تبعا لأسلوب اللغة المستعملة والبلد، وتقوم المجلات والصحف بوضع هذه اللعبة لتسلية القراء. ظهرت أول لعبة للكلمات المتقاطعة في صحيفة newyork world وذلك في 21 ديسمبر عام 1913، وأصبحت من الألعاب الرائجة في الولايات المتحدة، ومنها انتقلت إلى بقية دول العالم، وبمختلف اللغات، وكان أول من أدخلها إلى الصحافة هو آرثر وين.]]>
	</arabic>
 
 **/
import flash.external.ExternalInterface;
class com.xvisage.utils.StringUtils extends String {
	private var chars:String;
	private var tag:TextField;
	private var format:TextFormat;
	private var temp:TextField;
	private var htmlTags:Array = ["A", "B", "BR", "FONT", "IMG", "I", "LI", "P", "SPAN", "TEXTFORMAT", "U", "UL"];
	private var brackets:String = "(){}[]<>";
	private var specialChars:String = "اأإآدذرزوؤء";
	private var isMAC:Boolean;
	public var data:String;
	function StringUtils() {
		// check for OS, it seems like Flash Player behaves differently between Windows and Mac, especially for embedding fonts!
		isMAC = false;
		var version:String = getVersion();
		if (version.toLowerCase().indexOf("mac") != -1) {
			isMAC = true;
		}
		specialChars += String.fromCharCode(0xFEFC,0xFEFB,0xFEF8,0xFEF7,0xFEFA,0xFEF9,0xFEF6,0xFEF5);
	}
	/**
	 * @public
	 * creates proper arabic HTML string.
	 * 
	 * @param input String value of original arabic string.
	 * @param tag TextField target html enabled textfield.
	 * @return String value with correct arabic text.
	 */
	public function parseArabic(input:String, field:TextField):String {
		tag = field;
		format = tag.getTextFormat();
		data = input; // referencing original input string to use for matching/comparing instead of (TextField.text)
		var string:String = input;
		if (input.length>0) {
			var arabicChar:String = "";
			var arabicString:String = "";
			var latinChar:String = "";
			var latinString:String = "";
			var htmlString:String = "";
			chars = input;
			// format hard lines-breaks
			chars = chars.split("\r\n").join("\n");
			if (tag.multiline) {
				chars = chars.split("\r").join("<br />");
				chars = chars.split("\n").join("<br />");
			} else {
				chars = chars.split("\r").join("");
				chars = chars.split("\n").join("");
			}
			// remove diacritics from arabic characters (to enable diacritics we need to reconsider ligatures conditions after finding a way to render diacritics properly).
			chars = stripDiacritics();
			// temporarily create an autosized textfield off-stage to use for proper right to left wrapping
			// if using V2 Components somewhere, then use a static depth instead of tag._parent.getNextHighestDepth()
			temp = tag._parent.createTextField("temp", tag._parent.getNextHighestDepth(), -1000, -1000, format.size, format.size+2);
			temp.autoSize = true;
			if (tag.embedFonts) {
				temp.embedFonts = true;
			}
			temp.selectable = false;
			temp.html = true;
			temp.text = "";
			temp.data = "";
			var toggleHTML:Boolean = false;
			var toggleArabic:Boolean = false;
			var htmlLines:Array;
			var singleLine:Array = [];
			var multiLines:Array = [];
			// proper bi-directional with HTML wrapping
			var i:Number = 0;
			while (i<chars.length) {
				// detect begining of supported HTML tags
				if (chars.charAt(i-1) == ">" && toggleHTML) {
					toggleHTML = false;
				}
				// detect closing of supported HTML tags
				if (chars.charAt(i) == "<" && validateHTMLTag(i)) {
					toggleHTML = true;
				}
				if (chars.charAt(i) != " ") {
					// toggle Arabic characters
					if (validateArabicChar(i) || validateSymbol(i, toggleHTML)) {
						toggleArabic = true;
					} else {
						toggleArabic = false;
					}
				}
				// buildig text line
				if (!toggleArabic) {
					// look for Arabic characters while switching direction and build text line
					if (arabicString.length>0) {
						singleLine.push({arabic:true, html:false, value:arabicString});
						arabicString = "";
					}
				}
				if (toggleArabic || !toggleArabic && toggleHTML) {
					// look for non-Arabic characters while switching direction and build text line
					if (latinString.length>0) {
						singleLine.push({arabic:false, html:false, value:latinString});
						latinString = "";
					}
				}
				if (toggleArabic || !toggleArabic && !toggleHTML) {
					// look for HTML characters while switching direction and build text line
					if (htmlString.length>0) {
						htmlLines = splitHTMLLines(htmlString);
						if (htmlLines.length>0) {
							multiLines.push(singleLine);
							singleLine = [];
							for (var e:Number = 0; e<htmlLines.length; e++) {
								if (e>=0 && e<htmlLines.length-1) {
									multiLines.push([{arabic:false, html:true, value:htmlLines[e]}]);
								} else {
									singleLine.push({arabic:false, html:true, value:htmlLines[e]});
								}
							}
						} else {
							singleLine.push({arabic:false, html:true, value:htmlString});
						}
						htmlString = "";
					}
				}
				if (toggleArabic) {
					// generate proper Arabic character
					arabicChar = getCharState(i);
					// brackets symmetrical swapping
					arabicChar = symmetricalSwapping(arabicChar, i);
					// build proper Arabic string
					arabicString += arabicChar;
					// store string value to temporary TextField for later wrapping
					temp.data += arabicChar;
				} else {
					// generate non-Arabic character
					latinChar = chars.charAt(i);
					// build non-Arabic string
					if (toggleHTML) {
						htmlString += latinChar;
					} else {
						latinString += latinChar;
					}
					// store string value to temporary TextField for later wrapping
					temp.data += chars.charAt(i);
				}
				// assign string value to temporary TextField for later wrapping
				temp.htmlText = temp.data;
				temp.setTextFormat(format);
				// detect line-break for proper wrapping
				if (breakTextLine(i, toggleHTML)) {
					// build proper text line after line break detection
					if (arabicString.length>0) {
						singleLine.push({arabic:true, html:false, value:arabicString});
						arabicString = "";
					}
					if (htmlString.length>0) {
						// complete HTML tag before trimming end of line.
						if (chars.charAt(v) != ">") {
							var v:Number = i+1;
							while (v<chars.length) {
								htmlString += chars.charAt(v);
								if (chars.charAt(v) == ">") {
									break;
								}
								v++;
							}
							i = v;
						}
						htmlLines = splitHTMLLines(htmlString);
						if (htmlLines.length>0) {
							// trim end of line and move last open HTML tag to next line
							multiLines.push(singleLine);
							singleLine = [];
							for (var e:Number = 0; e<htmlLines.length; e++) {
								if (e>=0 && e<htmlLines.length-1) {
									multiLines.push([{arabic:false, html:true, value:htmlLines[e]}]);
								} else {
									singleLine.push({arabic:false, html:true, value:htmlLines[e]});
								}
							}
						} else {
							singleLine.push({arabic:false, html:true, value:htmlString});
						}
						htmlString = "";
					}
					if (latinString.length>0) {
						singleLine.push({arabic:false, html:false, value:latinString});
						latinString = "";
					}
					if (singleLine.length>0) {
						multiLines.push(singleLine);
						singleLine = [];
					}
					temp.text = "";
					temp.data = "";
				}
				i++;
			}
			// remove temporary TextField
			temp.removeTextField();
			// look for remaining characters build last text line
			if (arabicString.length>0) {
				singleLine.push({arabic:true, html:false, value:arabicString});
			}
			if (htmlString.length>0) {
				htmlLines = splitHTMLLines(htmlString);
				if (htmlLines.length>0) {
					multiLines.push(singleLine);
					singleLine = [];
					for (var e:Number = 0; e<htmlLines.length; e++) {
						if (e>=0 && e<htmlLines.length-1) {
							multiLines.push([{arabic:false, html:true, value:htmlLines[e]}]);
						} else {
							singleLine.push({arabic:false, html:true, value:htmlLines[e]});
						}
					}
				} else {
					singleLine.push({arabic:false, html:true, value:htmlString});
				}
			}
			if (latinString.length>0) {
				singleLine.push({arabic:false, html:false, value:latinString});
			}
			if (singleLine.length>0) {
				multiLines.push(singleLine);
			}
			string = "";
			// build multiline text block
			for (var v:Number = 0; v<multiLines.length; v++) {
				var hasHTML:Boolean = false;
				for (var e:Number = 0; e<multiLines[v].length; e++) {
					if (multiLines[v][e].html) {
						hasHTML = true;
						break;
					}
				}
				if (hasHTML) {
					autoCompleteHTMLTags(v, multiLines);
				}
			}
			for (var v:Number = 0; v<multiLines.length; v++) {
				var hasArabic:Boolean = false;
				for (var e:Number = 0; e<multiLines[v].length; e++) {
					if (multiLines[v][e].arabic) {
						hasArabic = true;
						break;
					}
				}
				if (hasArabic) {
					if (!isMAC || tag.embedFonts && isMAC) {
						// re-order text line string parts only for embedded fonts
						multiLines.splice(v, 1, reorderTextLine(multiLines[v]));
					} else {
						if (!multiLines[v][0].arabic) {
							// place to fix beginning text line with non-Arabic charatcers only for non-embedded fonts
						}
					}
				}
				for (var e:Number = 0; e<multiLines[v].length; e++) {
					if (multiLines[v][e].arabic) {
						// reverse Arabic characters for proper reading
						string += reverseChars(multiLines[v][e].value);
					} else {
						var baseChars:String = multiLines[v][e].value;
						// switch ending space character only for embedded fonts
						if (!isMAC || tag.embedFonts && isMAC) {
							if (baseChars.charAt(baseChars.length-1) == " " && !multiLines[v][e].html) {
								baseChars = " "+baseChars.substr(0, -1);
							}
						}
						string += baseChars;
					}
				}
				// using HTML line-break to join all generated text lines
				if (v<multiLines.length-1) {
					string += "\n";
				}
			}
		}
		// returning proper arabic text with the specified font
		return '<font face="'+format.font+'">'+string+'</font>';
	}
	/**
	 * @private
	 * helper method detects valid HTML tag.
	 * 
	 * @param i Number index of target character.
	 * @return Boolean value of valid HTML tag.
	 */
	private function validateHTMLTag(i:Number):Boolean {
		var htmlTAG:String = "";
		var v:Number = i+1;
		while (v<chars.length) {
			if (chars.charAt(v) == " " || chars.charAt(v) == ">") {
				break;
			} else {
				htmlTAG += chars.charAt(v);
			}
			v++;
		}
		var validHTMLTag:Boolean = false;
		for (v=0; v<htmlTags.length; v++) {
			if (htmlTAG.toLowerCase().indexOf(htmlTags[v].toLowerCase()) != -1) {
				validHTMLTag = true;
				break;
			}
		}
		return validHTMLTag;
	}
	/**
	 * @private
	 * helper method detects valid Arabic characters.
	 * 
	 * @param i Number index of target character.
	 * @return Boolean value of valid Arabic character.
	 */
	private function validateArabicChar(i:Number):Boolean {
		var valid:Boolean = false;
		var code:Number = chars.charCodeAt(i);
		if (code >= 1536 && code <= 1791 || code >= 1872 && code <= 1919 || code >= 64336 && code <= 65023 || code >= 65136 && code <= 65279) {
			valid = true;
		}
		return valid;
	}
	/**
	 * @private
	 * helper method detects valid symbols.
	 * 
	 * @param i Number index of target character.
	 * @param htmlOn Boolean if character resides in valid html tags.
	 * @return Boolean value of valid symbols.
	 */
	private function validateSymbol(i:Number, htmlOn:Boolean):Boolean {
		var valid:Boolean = false;
		var code:Number = chars.charCodeAt(i);
		if (code >= 33 && code <= 47 || code >= 58 && code <= 63 || code >= 123 && code <= 126) {
			// validate brackets symmetrical swapping
			var bidirectional:Object = validateBiDirectional(i, htmlOn);
			if (bidirectional.isBiDirectional || bidirectional.isArabic) {
				valid = true;
			}
		}
		return valid;
	}
	/**
	 * @private
	 * helper method detects bi-directional case.
	 * 
	 * @param i Number index of target character.
	 * @param htmlOn Boolean if character resides in valid html tags.
	 * @return Object values of bi-directional case.
	 */
	 private function validateBiDirectional(i:Number, htmlOn:Boolean):Object {
		var prevArabic:Boolean = false;
		var nextArabic:Boolean = false;
		var isBiDirectional:Boolean = false;
		var isArabic:Boolean = false;
		var htmlOff:Boolean = false;
		var v:Number;
		if (htmlOn) {
			if (i>0) {
				v = i-1;
				while (v>=0) {
					if (chars.charAt(v) == "<" && !htmlOff) {
						htmlOff = true;
					}
					if (chars.charAt(v) != " " && htmlOff) {
						if (validateArabicChar(v)) {
							prevArabic = true;
						}
						break;
					}
					v--;
				}
			}
			htmlOff = false;
			if (i<chars.length) {
				v = i+1;
				while (v<=chars.length) {
					if (chars.charAt(v) == ">" && !htmlOff) {
						htmlOff = true;
					}
					if (chars.charAt(v) != " " && htmlOff) {
						if (validateArabicChar(v)) {
							nextArabic = true;
						}
						break;
					}
					v++;
				}
			}
		} else {
			if (i>0) {
				v = i-1;
				while (v>=0) {
					if (chars.charAt(v) != " ") {
						if (validateArabicChar(v)) {
							prevArabic = true;
						}
						break;
					}
					v--;
				}
			}
			if (i<chars.length) {
				v = i+1;
				while (v<=chars.length) {
					if (chars.charAt(v) != " ") {
						if (validateArabicChar(v)) {
							nextArabic = true;
						}
						break;
					}
					v++;
				}
			}
		}
		if (!prevArabic && nextArabic || prevArabic && !nextArabic) {
			isBiDirectional = true;
		} else if (prevArabic && nextArabic) {
			isArabic = true;
		}
		return {isBiDirectional:isBiDirectional, isArabic:isArabic, prevArabic:prevArabic, nextArabic:nextArabic};
	 }
	/**
	 * @private
	 * symmetrical swapping to reserve bi-directional brackets
	 *
	 * @param char String value of target character.
	 * @param i Number index of target character.
	 * @return String value of new string with proper brackets
	 */
	private function symmetricalSwapping(char:String, i:Number):String {
		if (brackets.indexOf(char) != -1) {
			switch (char) {
				case "(":
				char = ")";
				break;
				case ")":
				char = "(";
				break;
				case "{":
				char = "}";
				break;
				case "}":
				char = "{";
				break;
				case "[":
				char = "]";
				break;
				case "]":
				char = "[";
				break;
				case "<":
				char = ">";
				break;
				case ">":
				char = "<";
				break;
			}
		}
		return char;
	}
	/**
	 * @private
	 * helper method detects line width for proper wrapping.
	 * 
	 * @param i Number index of target character.
	 * @param htmlOn Boolean if character resides in valid html tags.
	 * @return String value with reversed arabic characters.
	 */
	private function breakTextLine(i:Number, htmlOn:Boolean):Boolean {
		var isNewLine:Boolean = false;
		if (tag.multiline && chars.charAt(i) == " ") {
			var htmlOff:Boolean = false;
			var initHTML:String = temp.data;
			var htmlLineBreaks:Array = splitHTMLLines(initHTML);
			if (htmlLineBreaks.length>0) {
				initHTML = htmlLineBreaks[0];
			} else {
				var v:Number = i+1;
				while (v<chars.length) {
					initHTML += chars.charAt(v);
					if (htmlOn) {
						if (chars.charAt(v) == ">") {
							htmlOff = true;
						}
					} else {
						htmlOff = true;
					}
					if (chars.charAt(v) == " " && htmlOff) {
						break;
					}
					v++;
				}
			}
			temp.htmlText = initHTML;
			temp.setTextFormat(format);
			if (temp._width>=tag._width-(format.leftMargin ? format.leftMargin : 2)-(format.rightMargin ? format.rightMargin : 2)) {
				isNewLine = true;
			} else {
				temp.htmlText = temp.data;
			}
			temp.setTextFormat(format);
		}
		return isNewLine;
	}
	/**
	 * @private
	 * helper method for proper HTML tags syntax.
	 * 
	 * @param index Number order of text line.
	 * @param lines Array value of multiline text block generated object.
	 */
	private function autoCompleteHTMLTags(index:Number, lines:Array) {
		var prevLine:Array = lines[index-1];
		var currentLine:Array = lines[index];
		var openTag:String = "";
		var closeTag:String = "";
		var toggleCloseTag:Boolean = false;
		var i:Number;
		var v:Number;
		for (i=0; i<currentLine.length; i++) {
			if (currentLine[i].html && currentLine[i].value.indexOf("</") != -1) {
				v = 0;
				while (v<currentLine[i].value.length) {
					if (currentLine[i].value.charAt(v-2) == "<" && currentLine[i].value.charAt(v-1) == "/") {
						toggleCloseTag = true;
					}
					if (toggleCloseTag) {
						if (currentLine[i].value.charAt(v) == ">") {
							break;
						} else {
							closeTag += currentLine[i].value.charAt(v);
						}
					}
					v++;
				}
			}
			if (toggleCloseTag) {
				closeTag = closeTag.toLowerCase();
				break;
			}
		}
		if (toggleCloseTag) {
			for (i=prevLine.length-1; i>0; i--) {
				if (prevLine[i].html && prevLine[i].value.toLowerCase().indexOf("<"+closeTag+" ") != -1) {
					openTag = prevLine[i].value.toLowerCase().substring(prevLine[i].value.toLowerCase().lastIndexOf("<"+closeTag), prevLine[i].value.length);
					break;
				}
			}
			if (openTag.length>0) {
				prevLine.push({arabic:false, html:true, value:"</"+closeTag+">"});
				currentLine.unshift({arabic:false, html:true, value:openTag});
			}
		}
	}
	/**
	 * @private
	 * helper method for proper text line order in bi-directional case (only for embedded fonts).
	 * 
	 * @param line Array value of text line generated object.
	 * @return Array value of proper text line order.
	 */
	private function reorderTextLine(line:Array):Array {
		line.reverse();
		if (line.length>1) {
			for (var i:Number = 1; i<line.length-1; i++) {
				if (!line[i].html && line[i-1].html && line[i+1].html) {
					var prevTag:Object = line[i-1];
					var nextTag:Object = line[i+1];
					line.splice(i-1, 1, nextTag);
					line.splice(i+1, 1, prevTag);
				}
			}
		}
		return line;
	}
	/**
	 * @private
	 * helper method to reverse only arabic characters for proper right to left order.
	 * 
	 * @param input String value of original arabic string with HTML tags stripped out.
	 * @param tag TextField target html enabled textfield.
	 * @return String value with reversed arabic characters.
	 */
	private function reverseChars(input:String):String {
		var chars:Array = input.split("");
		if (!isMAC || tag.embedFonts && isMAC) {
			chars.reverse();
		}
		return chars.join("");
	}
	/**
	 * @private
	 * helper method splits HTML string into proper text lines.
	 * 
	 * @param input String value of HTML string.
	 * @return Array value of new text lines.
	 */
	private function splitHTMLLines(input:String):Array {
		if (tag.multiline) {
			if (input.indexOf("<BR />") != -1) {
				return input.split("<BR />");
			} else if (input.indexOf("<br />") != -1) {
				return input.split("<br />");
			} else if (input.indexOf("<bR />") != -1) {
				return input.split("<bR />");
			} else if (input.indexOf("<Br />") != -1) {
				return input.split("<Br />");
			} else if (input.indexOf("<BR>") != -1) {
				return input.split("<BR>");
			} else if (input.indexOf("<br>") != -1) {
				return input.split("<br>");
			} else if (input.indexOf("<bR>") != -1) {
				return input.split("<bR>");
			} else if (input.indexOf("<Br>") != -1) {
				return input.split("<Br>");
			} else {
				return [];
			}
		} else {
			return [];
		}
	}
	/**
	 * @private
	 * uses unicode character codes to get proper arabic characters with various cases.
	 * 
	 * @param chars String containing original text.
	 * @param i Number index of target character.
	 * @return String value with correct arabic character.
	 */
	private function getCharState(i:Number):String {
		var string:String;
		switch (chars.charAt(i)) {
			case "ا":
			string = setChar(i, String.fromCharCode(0x0627), String.fromCharCode(0x0627), String.fromCharCode(0xFE8E), String.fromCharCode(0xFE8E));
			break;
			case "أ":
			string = setChar(i, String.fromCharCode(0x0623), String.fromCharCode(0x0623), String.fromCharCode(0xFE84), String.fromCharCode(0xFE84));
			break;
			case "إ":
			string = setChar(i, String.fromCharCode(0x0625), String.fromCharCode(0x0625), String.fromCharCode(0xFE88), String.fromCharCode(0xFE88));
			break;
			case "آ":
			string = setChar(i, String.fromCharCode(0x0622), String.fromCharCode(0x0622), String.fromCharCode(0xFE82), String.fromCharCode(0xFE82));
			break;
			case "ب":
			string = setChar(i, String.fromCharCode(0x0628), String.fromCharCode(0xFE91), String.fromCharCode(0xFE92), String.fromCharCode(0xFE90));
			break;
			case "ت":
			string = setChar(i, String.fromCharCode(0x062A), String.fromCharCode(0xFE97), String.fromCharCode(0xFE98), String.fromCharCode(0xFE96));
			break;
			case "ث":
			string = setChar(i, String.fromCharCode(0x062B), String.fromCharCode(0xFE9B), String.fromCharCode(0xFE9C), String.fromCharCode(0xFE9A));
			break;
			case "ج":
			string = setChar(i, String.fromCharCode(0x062C), String.fromCharCode(0xFE9F), String.fromCharCode(0xFEA0), String.fromCharCode(0xFE9E));
			break;
			case "ح":
			string = setChar(i, String.fromCharCode(0x062D), String.fromCharCode(0xFEA3), String.fromCharCode(0xFEA4), String.fromCharCode(0xFEA2));
			break;
			case "خ":
			string = setChar(i, String.fromCharCode(0x062E), String.fromCharCode(0xFEA7), String.fromCharCode(0xFEA8), String.fromCharCode(0xFEA6));
			break;
			case "د":
			string = setChar(i, String.fromCharCode(0x062F), String.fromCharCode(0x062F), String.fromCharCode(0xFEAA), String.fromCharCode(0xFEAA));
			break;
			case "ذ":
			string = setChar(i, String.fromCharCode(0x0630), String.fromCharCode(0x0630), String.fromCharCode(0xFEAC), String.fromCharCode(0xFEAC));
			break;
			case "ر":
			string = setChar(i, String.fromCharCode(0x0631), String.fromCharCode(0x0631), String.fromCharCode(0xFEAE), String.fromCharCode(0xFEAE));
			break;
			case "ز":
			string = setChar(i, String.fromCharCode(0x0632), String.fromCharCode(0x0632), String.fromCharCode(0xFEB0), String.fromCharCode(0xFEB0));
			break;
			case "س":
			string = setChar(i, String.fromCharCode(0x0633), String.fromCharCode(0xFEB3), String.fromCharCode(0xFEB4), String.fromCharCode(0xFEB2));
			break;
			case "ش":
			string = setChar(i, String.fromCharCode(0x0634), String.fromCharCode(0xFEB7), String.fromCharCode(0xFEB8), String.fromCharCode(0xFEB6));
			break;
			case "ص":
			string = setChar(i, String.fromCharCode(0x0635), String.fromCharCode(0xFEBB), String.fromCharCode(0xFEBC), String.fromCharCode(0xFEBA));
			break;
			case "ض":
			string = setChar(i, String.fromCharCode(0x0636), String.fromCharCode(0xFEBF), String.fromCharCode(0xFEC0), String.fromCharCode(0xFEBE));
			break;
			case "ط":
			string = setChar(i, String.fromCharCode(0x0637), String.fromCharCode(0xFEC3), String.fromCharCode(0xFEC4), String.fromCharCode(0xFEC2));
			break;
			case "ظ":
			string = setChar(i, String.fromCharCode(0x0638), String.fromCharCode(0xFEC7), String.fromCharCode(0xFEC8), String.fromCharCode(0xFEC6));
			break;
			case "ع":
			string = setChar(i, String.fromCharCode(0x0639), String.fromCharCode(0xFECB), String.fromCharCode(0xFECC), String.fromCharCode(0xFECA));
			break;
			case "غ":
			string = setChar(i, String.fromCharCode(0x063A), String.fromCharCode(0xFECF), String.fromCharCode(0xFED0), String.fromCharCode(0xFECE));
			break;
			case "ف":
			string = setChar(i, String.fromCharCode(0x0641), String.fromCharCode(0xFED3), String.fromCharCode(0xFED4), String.fromCharCode(0xFED2));
			break;
			case "ق":
			string = setChar(i, String.fromCharCode(0x0642), String.fromCharCode(0xFED7), String.fromCharCode(0xFED8), String.fromCharCode(0xFED6));
			break;
			case "ك":
			string = setChar(i, String.fromCharCode(0x0643), String.fromCharCode(0xFEDB), String.fromCharCode(0xFEDC), String.fromCharCode(0xFEDA));
			break;
			case "ل":
			string = setChar(i, String.fromCharCode(0x0644), String.fromCharCode(0xFEDF), String.fromCharCode(0xFEE0), String.fromCharCode(0xFEDE));
			break;
			case "م":
			string = setChar(i, String.fromCharCode(0x0645), String.fromCharCode(0xFEE3), String.fromCharCode(0xFEE4), String.fromCharCode(0xFEE2));
			break;
			case "ن":
			string = setChar(i, String.fromCharCode(0x0646), String.fromCharCode(0xFEE7), String.fromCharCode(0xFEE8), String.fromCharCode(0xFEE6));
			break;
			case "ه":
			string = setChar(i, String.fromCharCode(0x0647), String.fromCharCode(0xFEEB), String.fromCharCode(0xFEEC), String.fromCharCode(0xFEEA));
			break;
			case "ة":
			string = setChar(i, String.fromCharCode(0x0629), "", "", String.fromCharCode(0xFE94));
			break;
			case "و":
			string = setChar(i, String.fromCharCode(0x0648), String.fromCharCode(0x0648), String.fromCharCode(0xFEEE), String.fromCharCode(0xFEEE));
			break;
			case "ؤ":
			string = setChar(i, String.fromCharCode(0x0624), String.fromCharCode(0x0624), String.fromCharCode(0xFE86), String.fromCharCode(0xFE86));
			break;
			case "ى":
			string = setChar(i, String.fromCharCode(0x0649), String.fromCharCode(0x0649), String.fromCharCode(0xFEF0), String.fromCharCode(0xFEF0));
			break;
			case "ي":
			string = setChar(i, String.fromCharCode(0x064A), String.fromCharCode(0xFEF3), String.fromCharCode(0xFEF4), String.fromCharCode(0xFEF2));
			break;
			case "ئ":
			string = setChar(i, String.fromCharCode(0x0626), String.fromCharCode(0xFE8B), String.fromCharCode(0xFE8C), String.fromCharCode(0xFE8A));
			break;
			case "ء":
			string = String.fromCharCode(0x0621);
			break;
			case "ـ":
			string = String.fromCharCode(0x0640);
			break;
			case "?":
			string = String.fromCharCode(0x061F);
			break;
			case ",":
			string = String.fromCharCode(0x060C);
			break;
			case ";":
			string = String.fromCharCode(0x061B);
			break;
			case "%":
			string = String.fromCharCode(0x066A);
			break;
			default:
			string = chars.charAt(i);
			break;
		}
		return string;
	}
	/**
	 * @private
	 * detects special cases for arabic ligatures.
	 * 
	 * @param chars String containing original text.
	 * @param i Number index of target character.
	 * @param solo String arabic character without joints.
	 * @param begin String arabic character with a trailing joint.
	 * @param middle String arabic character with both joint.
	 * @param end String arabic character with an initial joint.
	 * @return String value with correct arabic character.
	 */
	private function setChar(i:Number, solo:String, begin:String, middle:String, end:String):String {
		var string:String = "";
		// detect lam-alef (ﻻ) cases
		if (chars.charAt(i) == "ل" && chars.charAt(i+1) == "ا") {
			if (validateArabicChar(i-1) && specialChars.indexOf(chars.charAt(i-1)) == -1) {
				string = String.fromCharCode(0xFEFC);
			} else {
				string = String.fromCharCode(0xFEFB);
			}
			chars = chars.substring(0, i)+string+chars.substring(i+2, chars.length);
		} else if (chars.charAt(i) == "ل" && chars.charAt(i+1) == "أ") {
			if (validateArabicChar(i-1) && specialChars.indexOf(chars.charAt(i-1)) == -1) {
				string = String.fromCharCode(0xFEF8);
			} else {
				string = String.fromCharCode(0xFEF7);
			}
			chars = chars.substring(0, i)+string+chars.substring(i+2, chars.length);
		} else if (chars.charAt(i) == "ل" && chars.charAt(i+1) == "إ") {
			if (validateArabicChar(i-1) && specialChars.indexOf(chars.charAt(i-1)) == -1) {
				string = String.fromCharCode(0xFEFA);
			} else {
				string = String.fromCharCode(0xFEF9);
			}
			chars = chars.substring(0, i)+string+chars.substring(i+2, chars.length);
		} else if (chars.charAt(i) == "ل" && chars.charAt(i+1) == "آ") {
			if (validateArabicChar(i-1) && specialChars.indexOf(chars.charAt(i-1)) == -1) {
				string = String.fromCharCode(0xFEF6);
			} else {
				string = String.fromCharCode(0xFEF5);
			}
			chars = chars.substring(0, i)+string+chars.substring(i+2, chars.length);
		} else {
			// check for arabic character position in word (solo, begin, middle, end)
			if (i == 0) {
				if (specialChars.indexOf(chars.charAt(i)) != -1) {
					string = solo;
				} else {
					string = begin;
				}
			} else if (i == chars.length-1) {
				if (specialChars.indexOf(chars.charAt(i-1)) != -1) {
					string = solo;
				} else {
					string = end;
				}
			} else if (validateArabicChar(i-1) && validateArabicChar(i+1)) {
				if (specialChars.indexOf(chars.charAt(i-1)) != -1) {
					if (specialChars.indexOf(chars.charAt(i)) != -1) {
						string = solo;
					} else {
						string = begin;
					}
				} else {
					if (specialChars.indexOf(chars.charAt(i)) != -1 || chars.charAt(i+1) == "ء" || chars.charAt(i) == "ة") {
						if (chars.charAt(i-1) != "ة") {
							string = end;
						} else {
							string = begin;
						}
					} else {
						if (chars.charAt(i-1) != "ة") {
							string = middle;
						} else {
							string = begin;
						}
					}
				}
			} else {
				if (validateArabicChar(i-1) && !validateArabicChar(i+1)) {
					if (specialChars.indexOf(chars.charAt(i-1)) != -1) {
						string = solo;
					} else {
						string = end;
					}
				} else if (!validateArabicChar(i-1) && validateArabicChar(i+1)) {
					if (specialChars.indexOf(chars.charAt(i)) != -1) {
						string = solo;
					} else {
						string = begin;
					}
				} else if (!validateArabicChar(i-1) && !validateArabicChar(i+1)) {
					string = solo;
				}
			}
		}
		return string;
	}
	/**
	 * @private
	 * remove diacritics from arabic characters.
	 *
	 * @return String containing splitted characters without diacritics.
	 */
	 private function stripDiacritics():String {
		var string:String = "";
		var i:Number = 0;
		while (i<chars.length) {
			// arabic diacritics range from charcode 1611 and 1618
			if (chars.charCodeAt(i)<1611 || chars.charCodeAt(i)>1618) {
				string += chars.charAt(i);
			}
			i++;
		}
		return string;
	 }
	 /**
	 * @public
	 * creates proper arabic input field.
	 * 
	 * @param flashID String swf object id in HTML document.
	 * @param inputID String generated HTML input id.
	 * @param field TextField target text input in flash.
	 */
	public function createArabicInput(flashID:String, inputID:String, field:TextField):Void {
		tag = field;
		var point:Object = {x:tag._x, y:tag._y};
		tag.localToGlobal(point);
		ExternalInterface.call("createArabicInput", flashID, inputID, "", point.x, point.y, tag._width, tag._height, "multiline");
	}
	/**
	* @public
	* convert numeric characters
	* 
	* @param input Number numbers to be converted
	* @return String converted numbers
	*/
	public function convertNumbers(input:Number):String {
		var string:String = "";
		var i:Number = 0;
		while (i<String(input).length) {
			switch (String(input).charAt(i)) {
				case "0":
				string += String.fromCharCode(0x0660);
				break;
				case "1":
				string += String.fromCharCode(0x0661);
				break;
				case "2":
				string += String.fromCharCode(0x0662);
				break;
				case "3":
				string += String.fromCharCode(0x0663);
				break;
				case "4":
				string += String.fromCharCode(0x0664);
				break;
				case "5":
				string += String.fromCharCode(0x0665);
				break;
				case "6":
				string += String.fromCharCode(0x0666);
				break;
				case "7":
				string += String.fromCharCode(0x0667);
				break;
				case "8":
				string += String.fromCharCode(0x0668);
				break;
				case "9":
				string += String.fromCharCode(0x0669);
				break;
			}
			i++;
		}
		return string;
	}
}